'' =================================================================================================
''
''   File....... jm_ctrx_timer.spin
''   Purpose.... Timer based on idea from SLRM in Propeller forum
''   Author..... Jon "JonnyMac" McPhalen
''               Copyright (c) 2013 Jon McPhalen
''               -- see below for terms of use
''   E-mail..... jon@jonmcphalen.com
''   Started.... 
''   Updated.... 26 FEB 2013
''
'' =================================================================================================


con

  RX1 = 31                                                      ' programming / terminal
  TX1 = 30
  
  SDA = 29                                                      ' eeprom / i2c
  SCL = 28


con

  MS_2_DAYS = 1000 * 60 * 60 * 24                               ' ms in one day
  MS_2_HRS  = 1000 * 60 * 60                                    ' ms in one hour
  MS_2_MINS = 1000 * 60                                         ' ms in one minute
  MS_2_SECS = 1000                                              ' ms in one second


var

  long  inuse                                                   ' object started
  
  long  khz1pin                                                 ' pin used (outputs 1kHz)
  long  last                                                    ' last reading for elapsed

  
pub start(pin, preset)

'' Starts timer
'' -- pin will be set to output and generate 1kHz signal
'' -- ctra uses as 1kHz generator
'' -- ctrb used as 1ms timer/counter
'' -- if > 0, preset loaded into phsb

  stop

  frqa := ($8000_0000 / (clkfreq / 1_000)) << 1                 ' ticks for 1kHz
  ctra := (%00100 << 26) | pin                                  ' nco mode
  dira[pin] := 1                                                ' set to output

  frqb := 1
  if (preset > 0)                                               ' preset timer?
    phsb := preset
  else
    phsb := 0
  ctrb := (%01010 << 26) | pin                                  ' pos-edge mode
  
  khz1pin := pin                                                ' save pin for stop
  inuse := true
  

pub stop

'' Stop timer
'' -- phsb not cleared (allows restart with preset)

  if (inuse)
    dira[khz1pin] := 0
    ctrb := 0
    ctra := 0
    inuse := false
    

pub reset

'' Resets timer and elapsed ms sync point

  phsb := 0                                                     ' restart timer
  last := 0                                                     ' reset 


pub millis | ms

'' Return milliseconds since last reset

  ms := phsb                                                    ' get current ms
  if (ms < 0)                                                   ' if roll-over
    reset                                                       '  clear everything
    ms := 0

  return ms


pub elapsed | now, ms

'' Returns elapsed ms since last call or reset

  now := millis
  if (now > last)                                               ' etms okay?
    ms := now - last                                            '  yes, return delta
  else
    now := 0                                                    '  no, reset sync
    ms := 0

  last := now                                                   ' save new sync point
  
  return ms


pub time_str(ms)

'' Returns address of formatted string (dd hh:mm:ss.mmm")
'' -- constains z-string representation of running timer

  if (ms < 0)                                                   ' request current clock?
    ms := millis 

  lzdec((ms / MS_2_DAYS), @TimeFormat[0],  2)                   ' extract/format days
  ms //= MS_2_DAYS                                             
                                                                 
  lzdec((ms / MS_2_HRS), @TimeFormat[3], 2)                     ' extract/format hours
  ms //= MS_2_HRS                                              
                                                                   
  lzdec((ms / MS_2_MINS), @TimeFormat[6], 2)                    ' extract/format minutes
  ms //= MS_2_MINS                                             
                                                                 
  lzdec((ms / MS_2_SECS), @TimeFormat[9], 2)                    ' extract/format seconds
                                                              
  lzdec((ms // MS_2_SECS), @TimeFormat[12], 3)                  ' extract/format millis   
                                                             
  return @TimeFormat


pri lzdec(val, p_str, width) | divisor

'' Convert positive value to ASCII
'' -- with leading zeroes
'' -- for 0 to 99_999

  divisor := lookup(width: 1, 10, 100, 1_000, 10_000)
  val := val // (divisor * 10)

  repeat width
    byte[p_str++] := "0" + (val / divisor)
    val //= divisor 
    divisor /= 10


dat

TimeFormat      byte    "00 00:00:00.000", 0                    
  

dat

{{

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

}}