'' =================================================================================================
''
''   File....... jm_ctrx_timer_test.spin 
''   Purpose.... 
''   Author..... Jon "JonnyMac" McPhalen
''               Copyright (c) 2013 Jon McPhalen
''               -- see below for terms of use
''   E-mail..... jon@jonmcphalen.com
''   Started.... 
''   Updated.... 26 FEB 2013
''
'' =================================================================================================


con

  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000                                          ' use 5MHz crystal

  CLK_FREQ = ((_clkmode - xtal1) >> 6) * _xinfreq
  MS_001   = CLK_FREQ / 1_000
  US_001   = CLK_FREQ / 1_000_000


con

  RX1 = 31                                                      ' programming / terminal
  TX1 = 30
  
  SDA = 29                                                      ' eeprom / i2c
  SCL = 28


con 

  #(-1), IS_ON, IS_OFF

  #0, LSBFIRST, MSBFIRST
  

con

   #1, HOME, GOTOXY, #8, BKSP, TAB, LF, CLREOL, CLRDN, CR       ' PST formmatting control
  #14, GOTOX, GOTOY, CLS


obj

  term  : "fullduplexserial64"
  timer : "jm_ctrx_timer"
 

var


pub main | t, timestamp

  term.start(RX1, TX1, %0000, 115_200)
  pause(10)

  term.str(string(CLS, "Timer Demo", CR, CR))

  timer.start(16, false)                                        ' start timer, P16 = 1kHz

  t := cnt
  repeat
    timestamp := timer.millis                                   ' get current time
    term.rjdec(timestamp, 9, " ")
    term.str(string("    "))
    term.str(timer.time_str(timestamp))
    term.tx(CR) 
    waitcnt(t += (100 * MS_001))


con

  { --------------- }
  {                 }
  {   B A S I C S   }
  {                 }
  { --------------- }


pub pause(ms) | t

'' Delay program in milliseconds

  if (ms < 1)                                                   ' delay must be > 0
    return
  else
    t := cnt - 1776                                             ' sync with system counter
    repeat ms                                                   ' run delay
      waitcnt(t += MS_001)
    

pub high(pin)

'' Makes pin output and high

  outa[pin] := 1
  dira[pin] := 1


pub low(pin)

'' Makes pin output and low

  outa[pin] := 0
  dira[pin] := 1


pub toggle(pin)

'' Toggles pin state

  !outa[pin]
  dira[pin] := 1


pub input(pin)

'' Makes pin input and returns current state

  dira[pin] := 0

  return ina[pin]
  

con

  { ----------------- }
  {                   }
  {   S U P P O R T   }
  {                   } 
  { ----------------- }


pub bit_count(value) | bc

'' Returns # of bits set in value

  bc := 0                                                       ' clear count
  if (value)                                                    ' any bits?
    repeat 32                                                   '  yes, test all bits
      bc += (value & %1)                                        '  add bit value to count
      value >>= 1                                               '  next bit
     
  return bc


pub bit_pos(value, mode)

'' Returns position of 1st "1" bit
'' -- mode 0 (LSBFIRST) to scan from lsb, mode 1 (MSBFIRST) to scan from msb 
'' -- -1 = no bits set

  if (value == 0)                                               ' if no bits                                   
    return -1                                                   '  return -1

  else
    if (mode == LSBFIRST)                                       ' check from LSB
      value ><= 32                                              '  flip for >|
      return (32 - >|value)
    else
      return (>|value - 1)


pub rd_nib(target, pos)

'' returns nib value (0..15) of target.nib[pos]

  if ((pos => 0) and (pos =< 3))
    return (target >> (pos << 2)) & %1111
  else
    return 0

    
pub rd_quart(target, pos)

'' returns quarternary value (0..3) of target.quart[pos] 

  if ((pos => 0) and (pos =< 7))
    return (target >> (pos << 1)) & %11
  else
    return 0


pub rd_bit(target, pos)

'' returns bit value (0..1) of target.pos

  if ((pos => 0) and (pos =< 31))
    return (target >> pos) & 1
  else
    return 0


pub wr_bit(target, pos, value)

'' writes value.0 to target.pos

  if (value & 1)
    return target | (1 << pos)
  else
    return target & !(1 << pos)


pub toggle_bit(target, pos)

'' toggles target.pos

  return target ^ (1 << pos)


con

  { --------------------------- }
  {                             } 
  {   M A T H   S U P P O R T   }
  {                             }  
  { --------------------------- }
  
  
pub fix_deg(degrees)

'' Converts degrees (0 to 360) to 13-bit value for cos and sin

  return (degrees * 1024) / 45
  
  
pub cos(x)

'' Cosine of the angle x
'' -- 0 to 360 degrees == $0000 to $2000
'' -- returns -65536 (-1) to 65536 (+1)

  return sin(x + $0800)
  

pub sin(x) | value

'' Sine of the angle x
'' -- 0 to 360 degrees == $0000 to $2000
'' -- returns -65536 (-1) to 65536 (+1)


  if ((x & $0FFF) == $0800)
    value := $1_0000
  elseif (x & $0800)
    value := word[$E000][-x & $7FF]
  else
    value := word[$E000][x & $7FF]
    
  if (x & $1000)
    value := -value

  return value
    

con

  { --------------------------- }
  {                             } 
  {   C T R X   S U P P O R T   }
  {                             }   
  { --------------------------- }


pub set_freq(ctrx, px, fx)

'' Sets ctrx to frequency fx on pin px (NCO/SE mode)
'' -- fx in hz
'' -- use fx of 0 to stop counter that is running

  if (fx > 0)                             
    fx := ($8000_0000 / (clkfreq / fx)) << 1            ' convert freq for NCO mode    
    case ctrx
      "a", "A":
        ctra := ((%00100) << 26) | px                   ' configure ctra for NCO on pin
        frqa := fx                                      ' set frequency
        dira[px] := 1                                   ' make pin an output
     
      "b", "B":                         
        ctrb := ((%00100) << 26) | px  
        frqb := fx                    
        dira[px] := 1

  else
    case ctrx
      "a", "A":
        ctra := 0                                       ' disable counter
        outa[px] := 0                                   ' clear pin/driver 
        dira[px] := 0                                  
     
      "b", "B":                         
        ctrb := 0 
        outa[px] := 0  
        dira[px] := 0  


pri freqout_2x(p1, f1, p2, f2, killpntr)                ' launch with cognew()

'' Run one or both counters in NCO mode
'' -- designed to run in background cog

  if (p1 => 0)                                          ' output on this pin?
    ctra := ((%00100) << 26) | p1                       ' set counter for NCO on this pin (p1)
    frqa := ($8000_0000 / (clkfreq / f1)) << 1          ' set frqx for frequency (f1)
    dira[p1] := 1                                       ' set pin to output mode

  if (p2 => 0)
    ctrb := ((%00100) << 26) | p2 
    frqb := ($8000_0000 / (clkfreq / f2)) << 1
    dira[p2] := 1  

  long[killpntr] := false                               ' let cog run after setup
  repeat
  
    { let counter(s) run }
    
  until(long[killpntr])

  cogstop(cogid)                                        ' kill this cog
  
 
dat

{{

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

}}